/*
**
**  WAY (Where Are You ?)
**
**  main.c		main program module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  1.0.00   891030  Roberto Bagnara  First working version
**
**
**  Description:
**
**  this module contains the WAY main program.
**
*/

/*
**	Include files.
**	--------------
*/

#include <stdio.h>
#include <strings.h>
#include <rpcrts.h>
#include "way.h"

#ifndef DEFAULT_CLIENTS
#define DEFAULT_CLIENTS "RPC_CLIENT_NAME"
#endif

/*
**      External functions.
**      -------------------
*/

#ifdef __STDC__
extern void rpc_init(void);
extern void attach_way(void);
extern void attach_wayrc(void);
extern void rpc_loop_server(rpc_status *status, rpc_name client_name);
extern void rpc_report_error(rpc_status status);
extern void db_init(void);
extern int dbase_load_from_file(char *file_name);
extern char *version_string(void);
extern int atoi(char *nptr);
extern void exit(int status);
#else
extern void rpc_init();
extern void attach_way();
extern void attach_wayrc();
extern void rpc_loop_server();
extern void rpc_report_error();
extern void db_init();
extern int dbase_load_from_file();
extern char *version_string();
extern int atoi();
extern void exit();
#endif

/*
**      External functions that don't like too much checking.
**      -----------------------------------------------------
*/

extern void error(/* function, format, arg1, arg2, ... */);
extern void fatal(/* function, format, arg1, arg2, ... */);
extern void wlog(/* format, arg1, arg2, ... */);
extern void tlog(/* format, arg1, arg2, ... */);

/*
**	Global variables.
**	-----------------
*/

/*
**  Parser debug control variable.
**  ------------------------------
*/

#ifdef YYDEBUG
int yydebug = 1;
#endif

/*
**  The log file.
**  -------------
*/

FILE *log_file = 0;

/*
**  The database file name.
**  -----------------------
*/

char *dbase_file_name = 0;

/*
**  Address of remote WAY to use for initialization.
**  ------------------------------------------------
*/

char *init_address = 0;

/*
**  Information logging control variable.
**  -------------------------------------
**
**  Meaning:
**
**	0 no information logging;
**	1 log dbase changes;
**	2 extensive logging for debug purposes.
*/

int log_level = 0;

/*
**  Password required for some privileged remote control operations.
**  ----------------------------------------------------------------
*/

char password[MAX_PWD_LENGTH];

/*
**  Possibly wild-carded RPC address identifying the clients to be served.
**  ----------------------------------------------------------------------
*/

rpc_name clients_pattern;

/*
**	Keep on satisfying requests.
**	----------------------------
*/

#ifdef __STDC__
void serve(void)
#else
void serve()
#endif
{
    rpc_status status;

    /* Initialize the RPC system. */

    rpc_init();

    /* Declare the stubs. */

    attach_way();
    attach_wayrc();

    /* Loop until fatal error. */

    LOG(1) wlog("Entering server loop for clients `%s'.", clients_pattern);
    rpc_loop_server(&status, clients_pattern);

    /* Report any errors. */

    rpc_report_error(status);
}

static char *usage_string = "\
Usage: way [option ...] <clients>\n\
        -help              displays this usage summary;\n\
        -version           displays the WAY version in use;\n\
        -dbasefile <fname> selects the database file;\n\
        -logfile   <fname> selects the log file (- is stdout);\n\
        -loglevel  <n>     selects the amount of information logged;\n\
        -password  <pwd>   selects the password for privileged server use.\n\
";
        
/*
**	Display info about WAY command usage.
**	-------------------------------------
*/

#ifdef __STDC__
static void usage(void)
#else
static void usage()
#endif
{
    (void) fprintf(stderr, "%s", usage_string);
    exit(SUCCESS_EXIT_CODE);
}

/*
**	The main program.
**	-----------------
*/

int main(argc, argv, envp)
int argc;
char **argv;
char **envp;
{
    register int i;
    int status;
    char *log_file_name = 0;

    /* By default the password is null. */

    *password = '\0';

    /* No clients addresses specified yet. */

    *clients_pattern = '\0';

    for (i = 1; i < argc; i++)
	if (argv[i][0] == '-' && argv[i][1] != 0) {
	    register char *str = argv[i] + 1;

	    if (!strcmp(str, "logfile") || !strcmp(str, "l"))
		log_file_name = argv[++i];

	    else if (!strcmp(str, "init") || !strcmp(str, "i"))
		init_address = argv[++i];

	    else if (!strcmp(str, "password") || !strcmp(str, "p"))
		(void) strncpy(password, argv[++i], MAX_PWD_LENGTH);

	    else if (!strcmp(str, "help") || !strcmp(str, "h"))
		usage();

	    else if (!strcmp(str, "dbasefile") || !strcmp(str, "d"))
		dbase_file_name = argv[++i];

	    else if (!strcmp(str, "version") || !strcmp(str, "v"))
		fprintf(stderr, "WAY (Where Are You ?) version %s\n",
							    version_string());

	    else if (!strcmp(str, "loglevel") || !strcmp(str, "L")) {

		char *endp = str = argv[++i];

		while (*endp) {
		    if (*endp >= '0' && *endp <= '9')
			endp++;
		    else
			fatal("main", "invalid number `%s'", argv[i]);
		}
		log_level = atoi(str);
	    } else
		fatal("main", "invalid option `%s'", argv[i]);
	} else {
	    if (*clients_pattern != '\0')
		fatal("main", "this version supports only one clients pattern");
	    if (strlen(argv[i]) > RPC_NAME_LENGTH)
		fatal("main", "clients pattern `%s' too long (>%d)",
		    argv[i], RPC_NAME_LENGTH);
	    else
		strcpy(clients_pattern, argv[i]);
	}

    /* If information logging is enabled, open the log file. */

    if (log_file_name != (char *) 0 && log_level == 0)
        log_level = 1;

    if (log_level > 0) {
	if (log_file_name == (char *) 0)
	    log_file_name = DEFAULT_LOG_FILE;

	if (!strcmp(log_file_name, "-"))
    	    log_file = stdout;
    	else {
#ifdef HAS_FILE_IO
    		log_file = fopen(log_file_name, "a");
    		if (log_file == 0)
		    fatal("main", "cannot open the log file `%s'", log_file_name);
#else
    		fatal("main", "no file I/O, use `-l -' to send log info to stdout");
#endif /* HAS_FILE_IO */
    	}
	tlog("\n**********************************************");
	wlog(  "********** New WAY session started. **********");
	wlog(  "**********************************************\n");
    }

    /* Initialize the database to empty. */

    db_init();

#ifdef HAS_FILE_IO

    /* Check if database file has been specified, if not use the default. */

    if (dbase_file_name == (char *) 0)
	dbase_file_name = DEFAULT_DBASE_FILE;

    /* Try to initialize the database from file. */

    status = dbase_load_from_file(dbase_file_name);
    if (!status)
	error("main", "cannot initialize database from `%s'", dbase_file_name);

#endif /* HAS_FILE_IO */

    /* If the clients pattern has not been specified provide a default one. */

    if (*clients_pattern == '\0')
	strcpy(clients_pattern, DEFAULT_CLIENTS);

    /* Do the real job. */

    serve();

    /* Should never get here. */

    return SUCCESS_EXIT_CODE;
}
